#!/bin/sh

appname="smartdns"
INSTALL_DIR="/userdisk/appdata/${appname}"
port=535


set_iptable()
{
    IPS="`ifconfig | grep "inet addr" | grep -v ":127" | grep "Bcast" | awk '{print $2}' | awk -F: '{print $2}'`"
    for IP in $IPS
    do
            [ -z "$(iptables -t nat -S | grep "${appname}"-"$IP")" ] && iptables -t nat -A PREROUTING -p udp -d $IP --dport 53 -m comment --comment "${appname}"-"$IP" -j REDIRECT --to-ports ${port} 
    done

}

clear_iptable()
{
    eval `iptables -t nat -S | grep "${appname}"- | sed -e 's/-A/iptables -t nat -D/g' | sed -e 's/\$/;/g'`
}

set_dnsmasq() {
    [ ! -f "${INSTALL_DIR}/local.bak" ] && echo `uci get dhcp.@dnsmasq[0].local` > ${INSTALL_DIR}/local.bak
    [ ! -f "${INSTALL_DIR}/resolvfile.bak" ] && echo `uci get dhcp.@dnsmasq[0].resolvfile` > ${INSTALL_DIR}/resolvfile.bak
    uci set dhcp.@dnsmasq[0].local=127.0.0.1#$port
    [ ! -f "/tmp/resolv.conf.smart" ] && cp "${INSTALL_DIR}/resolv.conf" /tmp/resolv.conf.smart
    uci set dhcp.@dnsmasq[0].resolvfile=/tmp/resolv.conf.smart
    uci commit
    /etc/init.d/dnsmasq reload
}

reset_dnsmsq() {
    if [ -f "${INSTALL_DIR}/local.bak" ];then
        uci set dhcp.@dnsmasq[0].local=`cat ${INSTALL_DIR}/local.bak`
    else
        uci set dhcp.@dnsmasq[0].local=/lan/
    fi
    if [ -f "${INSTALL_DIR}/resolvfile.bak" ];then
        uci set dhcp.@dnsmasq[0].resolvfile=`cat ${INSTALL_DIR}/resolvfile.bak`
    else
        uci set dhcp.@dnsmasq[0].resolvfile=/tmp/resolv.conf.auto
    fi
    uci commit 
    /etc/init.d/dnsmasq reload
}

# set_default_dns() {
#     cat >> ${INSTALL_DIR}/${appname}.conf <<-EOF
# server-https https://doh.pub/dns-query
# server-tcp dot.pub:53
# server-https https://dns.alidns.com/dns-query
# server-tcp dns.alidns.com:53
# server-https https://cloudflare-dns.com/dns-query
# server-tcp 1dot1dot1dot1.cloudflare-dns.com:53
# server-https https://doh.opendns.com/dns-query
# server-tcp dot.sb:53
# server 202.96.128.166:53
# server 202.96.134.133:53
# EOF
# }

start() {

    [ -n "$(pidof ${appname})" ] && echo "【$appname】" "${appname}已经在运行！" && exit 1
    echo "【$appname】" "正在启动${appname}服务... "
    # 修改上游服务器
    # sed -i '/^server/d' ${INSTALL_DIR}/${appname}.conf
    # set_default_dns
    # 修改端口号
    sed -i '/bind \[::\]:/d' ${INSTALL_DIR}/${appname}.conf
    echo "bind [::]:${port}" >> ${INSTALL_DIR}/${appname}.conf
    echo "【$appname】" "添加dnsmasq上游解析dns中..."
    set_dnsmasq &> /dev/null
    [ $? -ne 0 ] && echo "【$appname】" "未监测到dnsmasq，尝试添加iptables规则..." && set_iptable

    ${INSTALL_DIR}/${appname} -c ${INSTALL_DIR}/${appname}.conf
    if [ $? -ne 0 ]; then
        echo "【$appname】" "启动${appname}服务失败！"
    else
        echo "【$appname】" "启动${appname}服务完成！"
    fi
        
}

stop() {

    echo "【$appname】" "正在停止${appname}服务... "
    
    killall -9 ${appname} &> /dev/null
    clear_iptable &> /dev/null
    reset_dnsmsq &> /dev/null
    # 

}

status() {

    if [ -n "$(pidof ${appname})" ]; then
            status="运行中|1"
    else
            status="未运行|0"
    fi
    echo "【$appname】" "${status}"
}

case "$1" in
    start) start ;;
    stop) stop ;;
    restart) stop; start ;;
    reload) clear_iptable && set_iptable ;;
    status) status ;;
esac
